@@ -33,8 +33,6 @@ bool address_matches(const char* mask, const struct sockaddr* addr, GError** err
 	int masklen;
 	int addrlen = addr->sa_family == AF_INET ? 4 : 16;
 
-	assert(addr->sa_family == AF_INET || addr->sa_family == AF_INET6);
-
 	strcpy(privmask, mask);
 
 	memset(&hints, 0, sizeof(hints));
@@ -55,6 +53,7 @@ bool address_matches(const char* mask, const struct sockaddr* addr, GError** err
 	}
 	aitmp = res;
 	while(res) {
+		assert(addr->sa_family == AF_INET || addr->sa_family == AF_INET6);
 		const uint8_t* byte_s;
 		uint8_t* byte_t;
 		uint8_t mask = 0;
@@ -236,16 +235,9 @@ uint64_t size_autodetect(int fhandle) {
 }
 
 int exptrim(struct nbd_request* req, CLIENT* client) {
-	/* Don't trim when we're read only */
-	if(client->server->flags & F_READONLY) {
-		errno = EINVAL;
-		return -1;
-	}
-	/* Don't trim beyond the size of the device, please */
-	if(req->from + req->len > client->exportsize) {
-		errno = EINVAL;
-		return -1;
-	}
+	/* Caller did range checking */
+	assert(!(client->server->flags & F_READONLY));
+	assert(req->from + req->len <= client->exportsize);
 	/* For copy-on-write, we should trim on the diff file. Not yet
 	 * implemented. */
 	if(client->server->flags & F_COPYONWRITE) {
