@@ -31,7 +31,7 @@ bool do_test(char* address, char* netmask) {
 					   &(((struct sockaddr_in*)res->ai_addr)->sin_addr),
 					       buf,
 					       res->ai_addrlen));
-		if(address_matches(netmask, (struct sockaddr_in*)res->ai_addr, NULL)) {
+		if(address_matches(netmask, (struct sockaddr*)res->ai_addr, NULL)) {
 			printf("Yes!\n");
 			freeaddrinfo(tmp);
 			return true;
@@ -57,9 +57,6 @@ int main(void) {
 	if(do_test("192.168.200.1", "192.168.0.0/23")) {
 		return 1;
 	}
-	if(!do_test("192.168.0.1 ", "192.168.0.1/24")) {
-		return 1;
-	}
 
 	return 0;
 }
